@@ -11,9 +11,12 @@ from h5.core import (
     datasets_dict as datasets_dict,
     datasets_gen as datasets_gen,
     datasets_gen_from_fspath as datasets_gen_from_fspath,
+    datasets_keys_list as datasets_keys_list,
+    fmt_h5_path as fmt_h5_path,
     groups as groups,
     groups_gen as groups_gen,
     groups_gen_from_fspath as groups_gen_from_fspath,
+    groups_keys_list as groups_keys_list,
     h5_attrs_dict as h5_attrs_dict,
     h5_attrs_gen as h5_attrs_gen,
     h5_attrs_gen_from_fspath as h5_attrs_gen_from_fspath,
@@ -25,12 +28,16 @@ from h5.core import (
     h5py_obj_dataset_gen as h5py_obj_dataset_gen,
     h5py_obj_gen as h5py_obj_gen,
     h5py_obj_groups_gen as h5py_obj_groups_gen,
+    h5py_obj_keys_gen as h5py_obj_keys_gen,
     is_dataset as is_dataset,
     is_file as is_file,
+    is_fspath as is_fspath,
     is_group as is_group,
+    is_group_like as is_group_like,
     is_h5py_dataset as is_h5py_dataset,
     is_h5py_file as is_h5py_file,
     is_h5py_group as is_h5py_group,
+    keys_list as keys_list,
 )
 
 __all__ = (
@@ -43,14 +50,16 @@ __all__ = (
     "attrs_dict",
     "attrs_gen",
     "attrs_gen_from_fspath",
-    "chain",
     "datasets",
     "datasets_dict",
     "datasets_gen",
     "datasets_gen_from_fspath",
+    "datasets_keys_list",
+    "fmt_h5_path",
     "groups",
     "groups_gen",
     "groups_gen_from_fspath",
+    "groups_keys_list",
     "h5_attrs_dict",
     "h5_attrs_gen",
     "h5_attrs_gen_from_fspath",
@@ -62,10 +71,14 @@ __all__ = (
     "h5py_obj_dataset_gen",
     "h5py_obj_gen",
     "h5py_obj_groups_gen",
+    "h5py_obj_keys_gen",
     "is_dataset",
     "is_file",
+    "is_fspath",
     "is_group",
+    "is_group_like",
     "is_h5py_dataset",
     "is_h5py_file",
     "is_h5py_group",
+    "keys_list",
 )
